# -*- coding: utf-8 -*

import os, sys, shutil, json
import IniParser
from Log.Logger import log2File
sys.path.append("./pycab")
from pycab.CabExtractor import CabExtractor

class FileMgr:
    def __init__(self, install_dir, update_data_dir, tmp_update_dir, tmp_updatecfg_conf, tmp_module_dat):
        self.install_dir = install_dir
        self.update_data_dir = update_data_dir
        self.tmp_update_dir = tmp_update_dir
        self.tmp_backup_dir = os.path.join(tmp_update_dir, 'backup')
        self.tmp_updatecfg_conf = tmp_updatecfg_conf
        self.tmp_module_dat = tmp_module_dat
        self.reboot = False
        self.isupdate = False

    def isExist(self, path):
        if os.path.exists(path):
            return True
        return False

    def copyToDir(self, srcFile, distDir):
        (filepath, filename) = os.path.split(srcFile)
        tempfilepath = distDir + filepath
        if os.path.exists(tempfilepath):
            shutil.rmtree(tempfilepath)
        os.makedirs(tempfilepath)
        shutil.copy(srcFile, tempfilepath)

    def setModuleIni(self, path_ini):

        iniParser = IniParser.IniParser(path_ini)
        iniParser.initconf()
        sections = iniParser.cf.sections()
        for section in sections:
            #for option in iniParser.cf.options(section):
            if iniParser.cf.has_option(section, 'path'):
                path = iniParser.get(section, "path")
                if path and 'productpath' in path:
                    new_path = path.replace("%productpath%", self.install_dir)
                    if not iniParser.set(section, 'path', new_path):
                        log2File(path_ini + " set productpath error")
                        return False
        return True

    def cmp_ver(self, ver1, ver2):
        lver1 = ver1.split('.')
        lver2 = ver2.split('.')
        f = 0
        if len(lver1) > len(lver2):
            f = len(lver1)
        else:
            f = len(lver2)
        for i in range(f):
            try:
                if int(lver1[i]) > int(lver2[i]):
                    return 1
                elif int(lver1[i]) == int(lver2[i]):
                    continue
                else:
                    return -1
            except IndexError as e:
                if len(lver1) > len(lver1):
                    return 1
                else:
                    return -1
        return 0

    def getModuleVer(self, module_name):
        mod_ver = ""
        with open(self.tmp_module_dat, 'r') as f:
            data = json.load(f)
            if not data.has_key("module_info"):
                log2File(self.tmp_module_dat + ':' +  'module_info is not exist.')
            else:
                module_info = data["module_info"]
                for module in module_info:
                    if module == module_name:
                        mod_ver = data["module_info"][module_name]["ver"]
                        log2File("asmodulize.dat " + module_name + ' version is ' + mod_ver)
                        break
        return mod_ver

    def updateModuleDat(self, module_name, version):
        with open(self.tmp_module_dat, 'r') as f:
            data = json.load(f)
            if not data.has_key("module_info"):
                log2File(self.tmp_module_dat + ':' +  'module_info is not exist.')
                return False
            else:
                module_info = data["module_info"]
                for module in module_info:
                    if module == module_name:
                        data["module_info"][module_name]["ver"] = version
                        log2File("update asmodulize.dat " + module_name + ' version is ' + version)
                        break

        with open(self.tmp_module_dat, 'wb') as f:
            #print data
            json.dump(data, f, indent=4)
        return True

    def updateModuleImpl(self, module_name, path_ini, module_dir):
        module_ini = IniParser.IniParser(path_ini)
        module_ini.initconf()
        update_ini = IniParser.IniParser(self.tmp_updatecfg_conf)
        update_ini.initconf()
        mod_ver = module_ini.get('360safe', 'libver')
        if mod_ver == "":
            log2File("get module " + module_name + " version fail.")
            return False
        mod_ver_old = self.getModuleVer(module_name)
        log2File(module_name + ":new version is " + mod_ver + ", old version is " + mod_ver_old)
        if mod_ver_old == "":
            log2File( module_name + " version is empty, no need update")
            return True
        if self.cmp_ver(mod_ver, mod_ver_old) <=0:
            log2File( module_name + " is no need update")
            return True
        file_str = module_ini.get(module_name, 'files0')
        log2File(module_name + '[' + file_str + ']')
        if file_str:
            file_list = file_str.split(',')
            for file in file_list:
                tmp_file = os.path.join(self.tmp_module_dir, file)
                ver_new = module_ini.get(file, 'ver')
                md5_new = module_ini.get(file, 'md5')
                dir_new = module_ini.get(file, 'path')
                if module_ini.cf.has_option(file, 'flag'):
                    flag = module_ini.get(file, 'flag')
                    if flag == "4":
                        self.reboot = True

                ver_old = ""
                if update_ini.cf.has_option(file.upper(), 'VER'):
                    ver_old = update_ini.get(file.upper(), 'VER')
                md5_old = ""
                if update_ini.cf.has_option(file.upper(), 'MD5'):
                    md5_old = update_ini.get(file.upper(), 'MD5')

                if dir_new:
                    if not os.path.exists(dir_new):
                        os.makedirs(dir_new)
                    file_new = os.path.join(dir_new, file)
                    if os.path.exists(file_new):
                        self.copyToDir(file_new, self.tmp_backup_dir)
                    else:
                        fp = open(os.path.join(self.tmp_update_dir, 'new_update_file'),'a')
                        fp.write(file_new)
                        fp.write('\n')
                        fp.close()

                if ver_old == "" or self.cmp_ver(ver_new, ver_old) >0 or (self.cmp_ver(ver_new, ver_old) == 0 and md5_new != md5_old):
                    cabfile = (file + '_' + ver_new).replace('.', '_') + '.cab'
                    self.extractFiles(cabfile, module_dir)
                    if os.path.exists(file_new) and os.path.exists(tmp_file):
                        os.remove(file_new)
                    shutil.copy(tmp_file, file_new)
                    log2File('update file:' + file_new + ', md5:' + md5_new)
                    self.isupdate = True
                    update_ini.set(file.upper(), 'VER', ver_new)
                    update_ini.set(file.upper(), 'PATH', file_new)
                    update_ini.set(file.upper(), 'MD5', md5_new)

        res = self.updateModuleDat(module_name, mod_ver)
        return res

    def extractFiles(self, filename , path_module_dir):
        path_cab_dir = self.tmp_module_dir
        extractor = CabExtractor()
        cab_file = os.path.join(path_module_dir, filename)
        if os.path.exists(cab_file):         
            log2File("Extract file " + cab_file)
            try:
                extractor.extract(cab_file)
                extractor.flush_to_disk(path_cab_dir)
            except:
                log2File("extract " + filename + " fail.")
                return False
        return True

    def updateModule(self, module_name, module_dir):
        res = False
        module_ini = module_name + '.ini'
        path_module_ini = os.path.join(module_dir, module_ini)
        path_module_dir = os.path.join(module_dir, module_name)
        if not self.isExist(path_module_ini):
            log2File(path_module_ini + " is not exits.")
            return res

        if not self.isExist(path_module_dir):
            log2File(path_module_dir + " is not exits.")
            return res

        shutil.copy(path_module_ini, self.tmp_module_dir)
        tmp_module_ini = os.path.join(self.tmp_module_dir, module_ini)
        #if not self.extractFiles(path_module_dir):
            #return res
        if not self.setModuleIni(tmp_module_ini):
            return res
        if not self.updateModuleImpl(module_name, tmp_module_ini, path_module_dir):
            return res

        res = True
        return res

    def endUpdate(self):
        if self.isupdate == False:
            return 0
        updatecfg_conf = os.path.join(self.install_dir, 'updatecfg.ini')
        if os.path.exists(updatecfg_conf):
            os.remove(updatecfg_conf)
            log2File('delete updatecfg.ini')
        shutil.copy(self.tmp_updatecfg_conf, updatecfg_conf)
        log2File('update updatecfg.ini')

        module_dat = os.path.join(self.install_dir, 'Data/asmodularize.dat')
        if os.path.exists(module_dat):
            os.remove(module_dat)
            log2File('delete asmodularize.dat')
        shutil.copy(self.tmp_module_dat, module_dat)
        log2File('update asmodularize.dat')

    def emptyTmpData(self):
        shutil.rmtree(self.tmp_update_dir)
        log2File('empty tmp update data directory')

    def update(self, module_name):
        res = False
        file_dir = os.path.join(self.update_data_dir, module_name)
        if not self.isExist(file_dir):
            file_dir = os.path.join(self.update_data_dir, 'virus', module_name)
            if not self.isExist(file_dir):
                log2File(module_name + ' is not exist.No need update')
                res = True
                return res
        self.tmp_module_dir = os.path.join(self.tmp_update_dir, module_name)
        if os.path.exists(self.tmp_module_dir):
            shutil.rmtree(self.tmp_module_dir)
        os.mkdir(self.tmp_module_dir)

        if not self.updateModule(module_name, file_dir):
            return res
        res = True
        return res

    def rollback(self):
        log2File('roolback backup files')
        for (root, dirs, files) in os.walk(self.tmp_backup_dir):
            for filename in files:
                file_path = os.path.join(root, filename)
                real_path = file_path.replace(self.tmp_backup_dir, "")
                if os.path.exists(real_path):
                    os.remove(real_path)
                shutil.copy(file_path, real_path)
                log2File('rollback file ' + real_path)
        delete_path = os.path.join(self.tmp_update_dir, 'new_update_file')
        if os.path.exists(delete_path):
            with open(delete_path, 'r') as f:
                for line in f.readlines():
                    path = line.strip()
                    if os.path.exists(path):
                        os.remove(path)
                        log2File('rollback delete ' + path)

    def rebootService(self):
        if self.reboot and self.isupdate:
            log2File("reboot 360safe service")
            if self.install_dir == "/opt/360safe":
                os.system('cat /opt/360safe/.360entclient.pid| xargs kill -9 >/dev/null 2>&1')
                os.system('service service360safe start >/dev/null 2>&1')
            else:
                os.system('cat /opt/360safeforcnos/.360safed.pid| xargs kill -9 >/dev/null 2>&1')
                os.system('service service360safe start >/dev/null 2>&1')

